from xaal.lib import tools,Device
from xaal import schemas
from . import devices

import atexit
import logging
import time

PACKAGE_NAME = 'xaal.tuya'
logger = logging.getLogger(__name__)

# poll devices only every n seconds
REFRESH_RATE=30

CFG_MAP = { 'powerrelay': devices.PowerRelay,
            'smartplug' : devices.SmartPlug }

class GW:
    def __init__(self,engine):
        self.engine = engine
        self.devices = {}
        atexit.register(self._exit)
        self.config()
        self.setup()
        engine.add_timer(self.update,5)
        
    def config(self):
        cfg = tools.load_cfg(PACKAGE_NAME)
        if not cfg:
            cfg= tools.new_cfg(PACKAGE_NAME)
            cfg['devices'] = {}
            logger.warn("Created an empty config file")
            cfg.write()
        self.cfg = cfg

    def setup(self):
        gw = schemas.devices.gateway(self.cfg['config']['addr'])
        gw.vendor_id  = 'IHSEV'
        gw.product_id = 'Generic Tuya Gateway'
        gw.info       = 'Tuya Gateway'
        gw.attributes['embedded'] = []
        self.gw = gw

        devices = self.cfg.get('devices',[])
        for d in devices:
            cfg        = devices.get(d,{})
            base_addr  = cfg.get('base_addr',None)
            ip         = cfg.get('ip',None)
            key        = cfg.get('key',None)
            type_      = cfg.get('type','PowerRelay')

            if base_addr == None:
                cfg['base_addr'] = base_addr = tools.get_random_uuid()[:-2]

            if ip and key:
                self.add_device(d,key,ip,base_addr,type_)
            else:
                logger.warn('Error in config file, wrong devices')
        # loaded all devices
        self.engine.add_device(gw)

    def add_device(self,tuya_id,tuya_key,ip,base_addr,type_):
        dev_type = CFG_MAP.get(type_,None)
        if dev_type:
            dev = dev_type(tuya_id,tuya_key,ip,base_addr)
            self.devices[tuya_id] = dev
            for d in dev.devices:
                self.engine.add_device(d)
                self.gw.attributes['embedded'].append(d.address)
            dev.update_status()
        else:
            logger.warn(f"Unsupported device type {type_} {ip}")
            
    def update(self):
        # Tuya protocol doesn't support update broadcasting
        # so we have to poll devices frequently to update the internal state
        # this is a pain, but if you don't use Smart Life App, this should never
        # occur anyway.
        now = time.time()
        for dev in self.devices.values():
            if now > (dev.last_update + REFRESH_RATE):
                dev.update_status()
                break # we only update one dev to avoid blocking
        
    def _exit(self):
        cfg = tools.load_cfg(PACKAGE_NAME)
        if cfg != self.cfg:
            logger.info('Saving configuration file')
            self.cfg.write()

def setup(eng):
    GW(eng)
    return True
